<?php

/**
 * Класс MG - предназначен для  доступа к функционалу системы,
 * из любой точки программы.
 * Реализован в виде синглтона, что исключает его дублирование.
 * Имеет в себе реестр для хранения любых объектов.
 *
 * @author Авдеев Марк <mark-avdeev@mail.ru>
 * @package moguta.cms
 * @subpackage Libraries
 */
class MG {

  static private $_instance = null;
  private $_registry = array();


  private function __construct() {
    // Старт сессии
    session_start();
    // Включение пользовательского шаблона
    self::enableTemplate();
    define('SITE', PROTOCOL.'://'.$_SERVER['SERVER_NAME'].URL::getCutSection());
  //  $this->_registry['staticPage'] = array('page' => 'html_content');
  }

  private function __clone() {

  }


  private function __wakeup() {

  }

  /**
   * Защита от XSS атак полученный массив параметров.
   *
   * @param array $arr параметры, которые надо защитить.
   * @return array $arr теже параметры, но уже безопасные.
   */
  public static function defenderXss($arr) {
    $filter = array('<', '>');

    foreach ($arr as $num => $xss) {
      $arr[$num] = str_replace($filter, '|', trim($xss));
    }

    return $arr;
  }


  /**
   * Отключает вывод элементов шаблона. Нужен при работе с AJAX.
   */
  public static function disableTemplate() {
    $_SESSION['noTemplate'] = true;
  }


  /**
   * Включает вывод элементов шаблона. Весь контент будет
   * выводиться внутри пользовательской темы оформления.
   */
  public static function enableTemplate() {
    $_SESSION['noTemplate'] = false;
  }


  /**
   * Возвращает переменную из реестра.
   * @param $key - имя перменной.
   */
  static public function get($key) {
    return !empty(self::getInstance()->_registry[$key])?self::getInstance()->_registry[$key]:null;
  }

  /**
   * Возвращает запрошенную настройку из памяти.
   * @param $option - имя перменной.
   */
  static public function getSetting($option) {    
    return !empty(self::getInstance()->_registry['settings'][$option])?self::getInstance()->_registry['settings'][$option]:null;
  }

  /**
   * Получает настройки для доступа к БД, из конфигурационного файла config.ini.
   * @return boolean.
   */
  public static function getConfigIni() {
    if (file_exists('config.ini')) {
      $config = parse_ini_file('config.ini', true);
      define('HOST', $config['DB']['HOST']);
      define('USER', $config['DB']['USER']);
      define('PASSWORD', $config['DB']['PASSWORD']);
      define('NAME_BD', $config['DB']['NAME_BD']);
      define('PREFIX', $config['DB']['TABLE_PREFIX']);
        // создание констант из настроек
      foreach ($config['SETTINGS'] as $key => $value) {
        define($key, $value);
      }
      return true;
    }
    return false;
  }


  /**
   * Получает контент статической HTML страницы из БД.
   * @todo Сделать поиск в статическом коде меток [имя_метки параметр1 параметр...]
   * @return string|boolean - возвращает либо HTML либо false.
   */
  public static function getHtmlContent() {
    $result = false;
    $arrayStaticPage = self::get('staticPage');
    
    foreach ($arrayStaticPage as $table => $content) {
      $res = DB::query('
      SELECT  *
      FROM '.PREFIX.$table.'
      WHERE url="'.URL::getRoute().'.html"
    ');

      if ($html = DB::fetchAssoc($res)) {

        $result = $html[$content];        
        self::titlePage($html['title']);
        self::seoMeta($html);

      }
    }
 
    return  $result;
  }


  /**
   * Возвращет единственный экземпляр данного класса.
   * @return object - объект класса MG.
   */
  static public function getInstance() {
    if (is_null(self::$_instance)) {
      self::$_instance = new self;
    }
    return self::$_instance;
  }

  /**
   * Получить путь до пользовательского файла, создающего контент страницы.
   * Файл должен находиться в папке mg-pages.
   * @return string - путь к php файлу.
   */
  public static function getPhpContent() {
    $result = false;

    if (file_exists(PAGE_DIR.URL::getRoute().'.php')) {
      $result = PAGE_DIR.URL::getRoute().'.php';
    }

    return  $result;
  }


  /**
   * Инициализация настроек сайта из таблицы settings в БД
   * Записывает в реестр все настройки из таблицы,
   * в последствии к ним осуществляется доступ из любой точки программы
   * @todo - скореее всего перепишем, а возмоно просто удалим, т.к. теперь есть опции.
   * @return void
   */
  public static function init() {
    
    $result = DB::query("
      SELECT `option`, `value`
      FROM `".PREFIX."setting`  
      ");
    $settings=array();
    
    while ($row = DB::fetchAssoc($result)) {
	
	  if(in_array($row['option'],array('imapHost','imapPort','imapLogin','imapPass','smtpHost','smtpLogin','smtpPass','smtpPort','mg_username','mg_password','mg_host'))){
	    $row['value'] = CRYPT::mgDecrypt($row['value']);
	  }
	  
      $settings[$row['option']] = $row['value'];
    }
    
    self::set('settings', $settings); // инициализируем объект MG попутно заполняя реестр натроек
    
   // include('mg-admin/locales/'.$settings['languageLocale'].'.php');
  //  self::set('lang', $lang);
    /**
     * Подключает файл адаптирующий вызовы статических методов, в обыкновенные функции.
     */
    require_once 'metodadapter.php';
    
    
    // Определяет константу PATH_TEMPLATE с путем до шаблона сайта.
    self::setDifinePathTemplate('default');

    // Подключает файл с функциями шаблона, если таковой существует.
    if (file_exists(PATH_TEMPLATE.'/functions.php')) {
      require_once PATH_TEMPLATE.'/functions.php';
    }
  }


  /**
   * Запускает инсталятор CMS.
   * @return void
   */
  public static function instalMoguta() {

    if (file_exists('install/install.php')) {
      require_once 'install/install.php';
      exit;
    } else {
      echo '<span>ВНИМАНИЕ!! Файл конфигурации недоступен!!
              Повторите процедуру инсталяции</span>';
      exit;
    }
  }


  /**
   * Функция Downtime (временное отключение работоспособности сайта).
   * @return boolean
   */
  public static function isDowntime() {
    $route = URL::getRoute();
  
    $role = USER::isAuth() ? USER::getThis()->role : '';
    $settings = self::get('settings');
    if ('mgadmin' != $route &&
      'ajax' != $route &&
      'enter' != $route &&
      $role!=1 &&
      'true' == $settings['downtime']) {
      return true;
    }
    return false;
  }


  /**
   * Функция проверяет наличие установленных библиотек PHP
   * @param type $mode тип проверки уставовленных модулей
   * @return boolean
   */
  public static function libExists($mode = 0) {
    $res = false;
    if (!function_exists('curl_init')) {
      $res[] = 'Пакет libcurl не установлен! Библиотека cURL не подключена.';
    }

    if (!extension_loaded('zip')) {
      $res[] = 'Пакет zip не установлен! Библиотека ZipArchive не подключена.';
    }

    file_put_contents('temp.txt', ' ');

    if (!file_exists('temp.txt')) {
      $res[] = 'Нет прав на создание файла. Загрузка архива с обновлением невозможна';
    } else {
      unlink('temp.txt');
    }
    return $res;
  }

  /**
   * Создает текстовый файл log_Y_m_d.txt в корневой папке сайта с отладочным содержанием. Можно использовать для логирования.
   * <code>
   * MG::loger("Любая строка");
   * MG::loger($var);
   * MG::loger($array); // для вывода в файл массива и полей объектов
   * </code>
   * @param string|array $text текст лога.
   * @param string $mode режим записи.
   * @param string $file название файла для записи.
   * @param string $time делать ли приписку дня в названии файла.
   * @param string $dump использовать ли var_dump.
   * @return void
   */
  public static function loger($text, $mode = 'apend', $file = 'log', $time = true, $dump = false) {
    switch ($mode) {
      case 'new':
      case 'w':
        $mode = 'w';
        break;
      case 'add':
      case 'apend':
      case 'append':
      case 'a+':
      default:
        $mode = 'a+';
        break;
    }
    $date = date('Y_m_d');
    $fileName = SITE_DIR.$file.($time ? '_'.$date : '').'.txt';
    ob_start();
    if($dump) {
      var_dump($text);
    } else {
      print_r($text);
    }
    $content = ob_get_contents();
    ob_end_clean();
    $content = str_replace('=>'."\n ", ' =>', $content);
    $string = date('d.m.Y H:i:s').' => '.$content."\r\n";
    $f = fopen($fileName, $mode);
    fwrite($f, $string);
    fclose($f);
    chmod($fileName, 0777);
  }

  /**
   * Возвращает созданую движком HTML страницу, для вывода на экран.
   * Имеет четыре типа вывода:
   * - представление из MVC;
   * - пользовательский php Файл;
   * - статическая HTML страница из БД;
   * - страница 404 ошибки, из пользовательского шаблона.
   *
   * @param mixed $data - массив с данными для вывода контента.
   * @return string - сгенерированный HTML код.
   */
  public static function printGui($data) {

    switch ($data['type']) {
      case 'view': {
          return self::getBuffer($data['view'], false, $data['variables']);
          break;
        }
      case 'php': {
          return self::getBuffer($data['data'], false);
          break;
        }
      case 'html': {
          self::set('isPage', true); // флаг статической страницы, если его нет, то страница формируется иным обазом
          return self::getBuffer($data['data'], true);
          break;
        }
      case '404': {
          header('HTTP/1.0 404 Not Found');
          self::titlePage('Ошибка 404');
          $path404 = PATH_TEMPLATE.'/404.php';

          if (!file_exists($path404)) {
            $path404 = 'mg-templates/default/404.php';
          }
          return self::getBuffer($path404);
          break;
        }
    }

    return false;
  }


  /**
   * Устанавливает meta данные страницы.
   * @param string|bool $title заголовок страницы.
   * @return void.
   */
  public static function meta() {
    $metaTitle = self::get('metaTitle');
    $metaKeywords  = self::get('metaKeywords');
    $metaDescription = self::get('metaDescription');

    $title = $metaTitle?$metaTitle:self::get('title');

    $meta = '
      <!--Заголовки определенные движком-->
      <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
      <title>'.$title.'</title>
      <meta name="keywords" content="'.$metaKeywords.'" />
      <meta name="description" content="'.$metaDescription.'" />

      <script type="text/javascript" src="'.SITE.'/mg-core/script/jquery-1.10.2.min.js"></script>';
 
      $meta .= '<!--/Заголовки определенные движком-->';
    $meta .= MG::get('userMeta');

  
    return  $meta;
  }


  /**
   * Создает переменную в реестре, в последствии доступна из любой точки программы
   * @param $key - имя перменной.
   * @param $object - значение переменной.
   */
  static public function set($key, $object) {
    self::getInstance()->_registry[$key] = $object;
  }


  /**
   * Устанавливает константу пути, до папки с шаблоном.
   * @param $template - папка с шаблоном в mg-templates/.
   */
  public static function setDifinePathTemplate($template = 'default') {
    $pathTemplate = 'mg-templates/'.$template;
    $path = $pathTemplate.'/css/style.css';

    if (!file_exists($path)) {
      $pathTemplate = 'mg-templates/default';
    }
    define('PATH_TEMPLATE', $pathTemplate);
    define('PATH_SITE_TEMPLATE', SITE.'/'.$pathTemplate);

    // Дописываем в includePath путь до шаблона,
    // для того чтобы дать влзможность изменять логику MVC локально, не трогая ядро.
    // Теперь  модель, вид и контролер в первую очередь
    // будут браться из пользовательского шаблона, при условии что они существуют.
    // *папки views, models и controlers - могут вовсе отсутствовать в шаблоне.

    set_include_path(PATH_TEMPLATE."/".PATH_SEPARATOR.get_include_path());

  }


  /**
   * Собирает массив данных, доступных в последствии из шаблоне через масиив $data.
   * @return void
   */
  public static function templateData($content) {

    $settings=self::get('settings');
    $data = array(
        'data' => array(       
        'content' => $content,      
        'thisUser' => User::getThis()
      )
    );

    return $data;
  }


  /**
   * Подключает пользовательский подвал сайта из выбранного шаблона.
   * Если футер в текущем шаблоне отсутствует поставляется стандартный код из шаблона default;
   * @todo - смутный функционал с подключением дефолтного футера.
   * @return void
   */
  public static function templateFooter($data = null) {
    $footerPath = PATH_TEMPLATE.'/footer.php';

    if (!file_exists($footerPath)) {
      $footerPath = 'mg-templates/default/footer.php';
    }

    if (!$_SESSION['noTemplate']) {
      require_once $footerPath;
    }
  }


  /**
   * Подключает пользовательскую шапку сайта из темы.
   * Если шапки в текущем шаблоне поставляется стандартный код из шаблона default;
   * @todo - смутный функционал с подключением дефолтного футера.
   * @return void
   */
  public static function templateHeader($data = null) {

    // делаем доступным массив $data в шаблоне.
    extract(self::templateData());
    include PATH_TEMPLATE.'/template.php';


    if (!$_SESSION['noTemplate']) {


      // Подключение админ панели.
      if (1 == User::getThis()->role) {
        require_once ADMIN_DIR.'/adminbar.php';
      }

      // Подключение файла шапки.
      require_once $headerPath;
    }
  }

    /**
   * Возвращает буфер, который содержит весь,
   * полученый в ходе работы движка, контент.
   * @param $include - путь для полкючаемого файла (вид или пользовательский файл).
   * @param $html - флаг, вывода html контента.
   * @param $variables - массив переменных, которые должны быть доступны в файле вида.
   */
  public static function getBuffer($include, $html = false, $variables = false) {

    if (!empty($variables)) {
      extract($variables);
    }
    ob_start();

    if ($html) {
      // выводим контент, предварительно заменив все шорткоды, результатами их обработки.
      echo $include;
    } else {
      // не подключается вид если view = _NONE_, например, открывается страница плагина.
      if ($include != '_NONE_') {
        include $include;
      }
    }

    $content = ob_get_contents();
    ob_end_clean();

    ob_start();
     // делаем доступным массив $data в шаблоне.
    if (!$_SESSION['noTemplate']) {
    self::printTemplate($content);
    }
    $buffer = ob_get_contents();
    ob_end_clean();

    // делаем доступным массив $data в шаблоне.
    if ($_SESSION['noTemplate']) {
      $buffer = $content;
    }

    return $buffer;
  }


  /**
   * Подключает пользовательскую шапку сайта из темы.
   * Если шапки в текущем шаблоне поставляется стандартный код из шаблона default;
   * @todo - смутный функционал с подключением дефолтного футера.
   * @return void
   */
  public static function printTemplate($content) {

    extract(self::templateData($content));
    include PATH_TEMPLATE.'/template.php';
     // Подключение админ панели.
     // if (1 == User::getThis()->role) {
     //   require_once ADMIN_DIR.'/adminbar.php';
     // }

  }

  /**
   * Задает заголовок страницы.
   * @return void
   */
  public static function titlePage($title) {  
    self::set('title', $title);
    // Инициализирует событие mg_titlePage. 
    // Чтобы обработать его пользховательской функцией нужно добавить обработчик:
    // mgAddAction('mg_titlepage', 'userFunctionName');
  }


  public static function seoMeta($data) {
    $data['meta_title'] = !empty($data['meta_title'])?$data['meta_title']:'';
    $data['meta_keywords'] = !empty($data['meta_keywords'])?$data['meta_keywords']:'';
    $data['meta_desc'] = !empty($data['meta_desc'])?$data['meta_desc']:'';
    
    self::set('metaTitle', $data['meta_title']);
    self::set('metaKeywords', $data['meta_keywords']);
    self::set('metaDescription', $data['meta_desc']);
    // Инициализирует событие mg_seoMeta.
    $args = func_get_args();
   
  }

 /**
   * Транслитирирует написания из кириллицу в латиницу.
   * @param string $str переводимая строка.
   * @param string $mode флаг для замены символа / на -.
   * @return string
   */
  public static function translitIt($str, $mode = 0) {
    $simb = '-';
    if ($mode == 1) {
      $simb = '/';
    }
    $tr = array(
      'А' => 'a',
      'Б' => 'b',
      'В' => 'v',
      'Г' => 'g',
      'Д' => 'd',
      'Е' => 'e',
      'Ё' => 'yo',
      'Ж' => 'j',
      'З' => 'z',
      'И' => 'i',
      'Й' => 'y',
      'К' => 'k',
      'Л' => 'l',
      'М' => 'm',
      'Н' => 'n',
      'О' => 'o',
      'П' => 'p',
      'Р' => 'r',
      'С' => 's',
      'Т' => 't',
      'У' => 'u',
      'Ф' => 'f',
      'Х' => 'h',
      'Ц' => 'ts',
      'Ч' => 'ch',
      'Ш' => 'sh',
      'Щ' => 'sch',
      'Ъ' => '',
      'Ы' => 'y',
      'Ь' => '',
      'Э' => 'e',
      'Ю' => 'yu',
      'Я' => 'ya',
      'а' => 'a',
      'б' => 'b',
      'в' => 'v',
      'г' => 'g',
      'д' => 'd',
      'е' => 'e',
      'ё' => 'yo',
      'ж' => 'j',
      'з' => 'z',
      'и' => 'i',
      'й' => 'y',
      'к' => 'k',
      'л' => 'l',
      'м' => 'm',
      'н' => 'n',
      'о' => 'o',
      'п' => 'p',
      'р' => 'r',
      'с' => 's',
      'т' => 't',
      'у' => 'u',
      'ф' => 'f',
      'х' => 'h',
      'ц' => 'ts',
      'ч' => 'ch',
      'ш' => 'sh',
      'щ' => 'sch',
      'ъ' => '',
      'ы' => 'y',
      'ь' => '',
      'э' => 'e',
      'ю' => 'yu',
      'я' => 'ya',
      '/' => $simb,
      '1' => '1',
      '2' => '2',
      '3' => '3',
      '4' => '4',
      '5' => '5',
      '6' => '6',
      '7' => '7',
      '8' => '8',
      '9' => '9',
      '0' => '0',
      'І' => 'i',
      'Ї' => 'i',
      'Є' => 'e',
      'Ґ' => 'g',
      'і' => 'i',
      'ї' => 'i',
      'є' => 'e',
      'ґ' => 'g',
      '«' => '',
      '» ' => '',
      ' ' => '-'
        
    );

    return strtr($str, $tr);
  }


  /**
   * Перенаправляет на другую страницу сайта.
   * @return void
   */
  public static function redirect($location, $redirect = '') {
    header('Location: '.$redirect.' '.SITE.$location);
    exit;
  }


  /**
   * Устанавливает значение для опции (настройки).
   * @param array $data -  может содержать значения для полей таблицы.
   * <code>
   * $data = array(
   *   option => 'идентификатор опции например: sitename'
   *   value  => 'значение опции например: moguta.ru'
   *   active => 'в будущем будет отвечать за автоподгрузку опций в кеш Y/N'
   *   name => 'Метка для опции например: Имя сайта'
   *   desc => 'Описание опции: Настройа задает имя для сайта'
   * )
   * </code>
   * @return void
   */
  public static function setOption($data) {
    // Если функция вызвана вот так: setOption('option', 'value');
    if (func_num_args() == 2) {
      $arg = func_get_args();
      $data = array();
      $data['option'] = $arg[0];
      $data['value'] = $arg[1];
    }

    $result = DB::query("
    SELECT *
    FROM `".PREFIX."setting`
    WHERE `option` = '%s'
    ", $data['option']
    );

    if (!DB::numRows($result)) {
      $result = DB::query("
      INSERT INTO `".PREFIX."setting`
      VALUES ('','%s','','N','')"
          , $data['option']);
    }

    $result = DB::query("
    UPDATE `".PREFIX."setting`
    SET ".DB::buildPartQuery($data)."
    WHERE `option` = '%s'
    ", $data['option']
    );
  }


  /**
   * Возвращает значение для запрошенной опции (настройки).
   * имеет два режима:
   * 1. getOption('optionName') - вернет только значение;
   * 2. getOption('optionName' , true) - вернет всю информацию об опции в
   * виде массива.
   * <code>
   * $data = array(
   *   option => 'идентификатор опции например: sitename'
   *   value  => 'значение опции например: moguta.ru'
   *   active => 'в будущем будет отвечать за автоподгрузку опций в кеш Y/N'
   *   name => 'Метка для опции например: Имя сайта'
   *   desc => 'Описание опции: Настройа задает имя для сайта'
   * )
   * </code>
   * @return void
   */
  public static function getOption($option, $data = false) {

    // Если функция вызвана вот так: getOption('option', true);
    if ($data) {
      $result = DB::query("
      SELECT *
      FROM `".PREFIX."setting`
      WHERE `option` = ".DB::quote($option)."
      "
      );
      if ($option = DB::fetchAssoc($result)) {
        return $option;
      }
    }

    $result = DB::query("
      SELECT value
      FROM `".PREFIX."setting`
      WHERE `option` = ".DB::quote($option)."
    ");

    if ($option = DB::fetchAssoc($result)) {
      return $option['value'];
    }
  }

  /**
   *  Склонение числительных
   *  пример echo 'Найдено '.declensionNum(5, array('иностранный язык', 'иностранных языка', 'иностранных языков'));
   * @param type $number
   * @param type $titles
   * @return type
   */
  public static function declensionNum($number, $titles){      
      $cases = array (2, 0, 1, 1, 1, 2);
      return $number." ".$titles[ ($number%100 > 4 && $number %100 < 20) ? 2 : $cases[min($number%10, 5)] ];
  }

   /**
   * Возвращает html для построения списка выбора количества записей
   * @param string $section = 'listemail'
   */
  public static function getHtmlCountRows($array=array(5,10,20,30,50,80,100,300)){

   return '<select name="countRows">
        <option value="1">1</option>
        <option value="2">2</option>
        <option value="3">3</option>
        <option value="4">4</option>
        <option value="5">5</option>
        <option value="10">10</option>
        <option value="20">20</option>
    </select>';
  }
  /**
   *  Возвращает количество записей для вывода в определенной секции админки 
   * @param string $section = 'listemail'
   */
  public static function getCountRows($section){
    $countRows = self::validCountRows();
    return $countRows[$section];
  }
  
   /**
   * Записывает количество записей для вывода в определенной секции админки 
   * @param string $section = 'listemail'
   * @param int $count = '10'
   * $count
   */
  public static function setCountRows($section,$count){
    $countRows = self::validCountRows();
    $countRows[$section]=$count;
    MG::setOption('countRows', addslashes(serialize($countRows)));
    return true;
  }
  
  /**
   * Проверяет целостность опции countRows в базе
   * возвращает массив countRows
   */
  public static function validCountRows(){

    $countRows = MG::getSetting('countRows');
    $countRows = unserialize(stripslashes($countRows));

    $validRows = array(
      'autoresponder'=>25,
      'emailbase'=>25,
      'history'=>25,
      'letter'=>25,
      'listemail'=>25,
      'rss'=>25,
      'task'=>25);

    if(!is_array($countRows)){
      $countRows = $validRows;
      MG::setOption('countRows', addslashes(serialize($countRows)));
      return $countRows;
    }

    foreach($validRows as $key=> $value){
      if(empty($countRows[$key])){
        $countRows[$key] = $value;
      }
    }
    return $countRows;
  }

  /**
   * Возвращает выбираемый HTML список количества записей на странице
   * в соответствии с требуемой секцией алминки
   */
  public static function getSelecterCountRows($section){
    $html = '<select class="countRowsToSection" data-section="'.$section.'">';
    $countForSection = self::getCountRows($section);
    foreach (array(3, 5, 10, 20, 25, 50, 100, 300) as $value) {
      $selected = '';
      if ($value == $countForSection) {
        $selected = 'selected="selected"';
      }
      $html .= '<option value="' . $value . '" ' . $selected . ' >' . $value . '</option>';
    }
    $html .= '</select>';
    return $html;

  }

  /**
   * Рекурсивно перемещает папку и содержимое.
   * @access private
   * @param string $source путь к исходной папке
   * @param string $dest путь куда перемещать папку
   * @param bool $copy при значении true копирует папку, а не перемещает
   */
  public static function rMoveDir($source, $dest, $copy = false) {
    mkdir($dest, 0755);
    if (!is_dir($source) || !is_dir($dest)) {return false;}
    foreach (
      $iterator = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($source, RecursiveDirectoryIterator::SKIP_DOTS),
        RecursiveIteratorIterator::SELF_FIRST) as $item
        ) {
      if ($item->isDir()) {
        mkdir($dest.DS.$iterator->getSubPathName());
      }
      else {
        if ($copy) {
          copy($item, $dest.DS.$iterator->getSubPathName());
        }
        else{
          rename($item, $dest.DS.$iterator->getSubPathName());
        }
      }
    }
    if (!$copy) {
      MG::rrmdir($source);
    }
  }

  /**
   * Рекурсивно удаляет папку и содержимое.
   * 
   * <code>
   *  // удалить все картинки первой сотни товаров, но оставить папки
   *  $dir = SITE_DIR.'uploads'.DS.'product'.DS.'000';
   *  MG::rrmdir($dir, false);
   * </code>
   * @access private
   * @param string $dir путь к папке для удаления
   * @param bool $deleteFolders удалять сами папки
   */
  public static function rrmdir($dir, $deleteFolders = true) { 
    if(is_dir($dir)) {
      $objects = array_diff(scandir($dir), array('.', '..')); 
      foreach ($objects as $object) { 
        if(is_dir($dir.DS.$object)) {
          self::rrmdir($dir.DS.$object, $deleteFolders);
        } else {
          unlink($dir.DS.$object); 
        }
      }
      if($deleteFolders) {
        rmdir($dir);
      }
    } 
  }
}